//-----------------------------------------------------------------------------
// MK3_2560_WAV
// Version 1.2
// 15.07.2012 / UB
// "www.Mikrocontroller-4U.de"
// mc-4u(@)t-online.de
//-----------------------------------------------------------------------------
// Fuer das myAVR MK3 Board
// mit ATMega-2560 @ 16MHz
// WAV-Ausgabe
// Ausgabe von normalen WAV-Files (vom Typ PCM) per PWM
// Die Wave-Files werden von der SD-Karte geladen
//-----------------------------------------------------------------------------
// das File muss im Ordner
// "myAVR/WinAVR/AVR/Include/UB_MK3_2560/"
// liegen und mit #include <UB_MK3_2560/MK3_2560_WAV.h>
// eingebunden werden
//-----------------------------------------------------------------------------
// Belegt : Port-B Bit-7 (linker Kanal)
//          Port-G Bit-5 (rechter Kanal) [bei Mono = linker Kanal]
//          Timer-0 (8bit)
//          Timer-1 (16bit)
// Benutzt : MK3_2560_FASTFAT16.h = SD-Karte = Port-B (Bit0 bis Bit3)
//-----------------------------------------------------------------------------
//
// Hinweis :
//
// es wird ein Mono (bzw Stereo) 8bit PWM-Signal erzeugt (mit 62,5kHz)
//
// Max Samplingrate ohne das zuviel Stoerungen zu hoehren sind
//   Bei 8bit Mono    = 48kHz
//   Bei 8bit Stereo  = 32kHz
//   Bei 16bit Mono   = 32kHz
//   Bei 16bit Stereo = 16kHz
//
// An den PWM-Ausgang sollte ein Filter angeschlossen werden
// um das Rechtecksignal etwas zu glaetten
//  z.B. RC-Filter 1.Ordnung (1k und 1u)
//-----------------------------------------------------------------------------

#include "UB_MK3_2560/MK3_2560_FASTFAT16.h"
#include <avr/interrupt.h>

//-----------------------------------------------------------------------------
// Hardware Settings
//-----------------------------------------------------------------------------
#define WAV_QUARZ_FRQ  16000000    // 16MHz


//-----------------------------------------------------------------------------
// Defines
//-----------------------------------------------------------------------------
#define WAV_MAX_SAMPLE 48000   // maximale sampling rate (22100 = 22,1kHz)
#define WAV_BUFFER_LEN 512     // laenge vom WAV-Puffer in Bytes (512 = optimal)
                               // Hinweis : es werden zwei Puffer angelegt
                               // Wichtig : die Zahl muss durch 4 ohne Rest teilbar sein


//-----------------------------------------------------------------------------
// INFOS zur Benutzung :
// 01. Funktion "MK3_INIT_WAV"  initialisiert den WAV-Treiber
// 02. Funktion "MK3_WAV_LOAD"  zum laden eines WAV-Files von der SD-Karte
// 03. Funktion "MK3_WAV_START" zum starten eines geladenen WAV-Files
// 04. Funktion "MK3_WAV_PAUSE" schaltet in den Pausemode
// 05. Funktion "MK3_WAV_STOP"  zum stoppen eines WAV-Files
// 06. Funktion "MK3_WAV_DO"    muss staendig von der Main aufgerufen werden
//-----------------------------------------------------------------------------


//-----------------------------------------------------------------------------
// Enumerationen fuer die Funktionen
//-----------------------------------------------------------------------------

typedef enum {
  WAV_LOAD_OK = 0,    // alles ok
  WAV_NO_STOP,        // status nicht auf STOP
  WAV_NO_SD,          // keine SD-Karte gefunden
  WAV_PARTITION_ERR,  // Fehler beim oeffnen der Partition
  WAV_FAT_ERR,        // Fehler beim oeffnen der FAT
  WAV_DIR_ERR,        // Fehler beim oeffnen der Direktory
  WAV_FILE_NOT_FOUND, // WAV-File nicht gefunden
  WAV_HEADER_LEN_ERR, // Fehler in der WAV-Header-Laenge
  WAV_HEADER_ERR      // Fehler im WAV-Header (siehe Fehlernr)
}WAV_CHK;

typedef enum {
  WAV_STOP =0,        // STOP
  WAV_PAUSE,          // PAUSE
  WAV_PLAY            // PLAY
}WAV_STATUS;

typedef enum {
  WAV_1MONO =0,       // 1 Frame Mono
  WAV_2MONO,          // 2 Frames Mono
  WAV_1STEREO,        // 1 Frame Stereo
  WAV_2STEREO         // 2 Frames Stereo
}WAV_MODE;

typedef enum {
  WAV_PCM_NORMAL =0,  // normales PCM Format
  WAV_PCM_FACT        // PCM mit "fact-chunk"
}WAV_TYP;


//-----------------------------------------------------------------------------
// Die Daten aus dem Header vom WAV-File werden in eine Struktur gelesen
//-----------------------------------------------------------------------------
typedef struct {
  bool header_ok;	  // true, wenn WAV-Header ok ist
  unsigned char err_nr;   // error nummer bei fehler
  unsigned int fmt_tag;   // TAG vom WAV-File (0001 = PCM)
  unsigned int kanal;     // Anzahl der Kanaele (1=Mono, 2=Stereo)
  unsigned int sampl;     // sample Rate (Lo-Wert) [8khZ = 8000]
  unsigned int frame;     // frame groesse (1,2,4)
  unsigned int bits;      // bits pro sample (z.B. 8)
  unsigned long len;      // anzahl der musik daten
  WAV_MODE mode;          // modus 
  WAV_STATUS status;      // aktueller status
}WAV_t;

//-----------------------------------------------------------------------------
// globale Variablen
//-----------------------------------------------------------------------------
WAV_t WAV_HEADER;		// hier stehen die Header-Daten drinn



//-----------------------------------------------------------------------------
// Funktionen die vom MAIN benutzt werden koennen
//-----------------------------------------------------------------------------
void MK3_INIT_WAV(void);	       // zum initialisieren
WAV_CHK MK3_WAV_LOAD(const char *ptr); // zum laden eines WAV-Files
void MK3_WAV_START(const char *ptr);   // zum starten eines WAV-Files
void MK3_WAV_PAUSE(void);              // zum pausieren eines WAV-Files
void MK3_WAV_STOP(void);               // zum stopen eines WAV-Files
void MK3_WAV_DO(void);                 // Do-Funktion (muss in die Main-while)





//-----------------------------------------------------------------------------
// Port-Pin definition
//-----------------------------------------------------------------------------
#define MK3_DDR_WAV_L DDRB	// DDR-Register der Ton-Ausgabe (links)
#define MK3_DDR_WAV_R DDRG	// DDR-Register der Ton-Ausgabe (rechts)

#define MK3_WAV_PIN_L  0x80     // Pin vom linken Kanal  (mono+stereo)
#define MK3_WAV_PIN_R  0x20     // Pin vom rechten Kanal (stereo)


//-----------------------------------------------------------------------------
// interne Funktionen
//-----------------------------------------------------------------------------
WAV_CHK WAV_LOAD_FILE(const char *ptr, WAV_TYP typ);
void WAV_CHK_HEADER(WAV_TYP typ);

//-----------------------------------------------------------------------------
// interne globale Variablen
//-----------------------------------------------------------------------------



typedef struct {
  uint8_t buffer[2][WAV_BUFFER_LEN]; // zwei puffer einrichten
  unsigned int akt_pos;
  unsigned int akt_buffer;
  bool load_buffer;
  bool file_ende;
  bool file_stop;
}WAV_DATA_t;
WAV_DATA_t WAV_DATA; // hier stehen die Musikdaten drinn




//-----------------------------------------------------------------------------
// Ueberpruefungen
//-----------------------------------------------------------------------------
#if WAV_QUARZ_FRQ != 16000000
  // WICHTIG !! Quarz-Frq MUSS 16MHz sein
  #error "WAV_QUARZ_FRQ wrong value" 
#endif 

#if WAV_BUFFER_LEN < 100
  // WICHTIG !! Laenge vom WAV-Puffer muss > 100 sein
  #error "WAV_BUFFER_LEN wrong value"
#endif

#if (WAV_BUFFER_LEN & 0x01) > 0
  // WICHTIG !! Laenge vom WAV-Puffer muss durch 4 teilbar sein
  #error "WAV_BUFFER_LEN Bit0 set"
#endif

#if (WAV_BUFFER_LEN & 0x02) > 0
  // WICHTIG !! Laenge vom WAV-Puffer muss durch 4 teilbar sein
  #error "WAV_BUFFER_LEN Bit1 set"
#endif

#if WAV_MAX_SAMPLE > 60000
  // WICHTIG !! Samplerate muss kleiner als 60000 sein
  #error "WAV_MAX_SAMPLE wrong value"
#endif


//-----------------------------------------------------------------------------
void MK3_INIT_WAV(void) {	// zum initialisieren der Schalter
  unsigned int n=0;

  // Header loeschen
  WAV_HEADER.header_ok=false;
  WAV_HEADER.err_nr=0;
  WAV_HEADER.fmt_tag=0;
  WAV_HEADER.kanal=0;
  WAV_HEADER.sampl=0;
  WAV_HEADER.frame=0;
  WAV_HEADER.bits=0;
  WAV_HEADER.len=0;
  WAV_HEADER.mode=WAV_1MONO;  
  WAV_HEADER.status=WAV_STOP;

  // Daten loeschen
  for(n=0;n<WAV_BUFFER_LEN;n++) {
    WAV_DATA.buffer[0][n]=0;
    WAV_DATA.buffer[1][n]=0;
  }
  WAV_DATA.akt_pos=0;
  WAV_DATA.akt_buffer=0;
  WAV_DATA.load_buffer=false;
  WAV_DATA.file_ende=false;
  WAV_DATA.file_stop=false;

  // PWM-Pins als Ausgang definieren
  MK3_DDR_WAV_L |= MK3_WAV_PIN_L;
  MK3_DDR_WAV_R |= MK3_WAV_PIN_R;

  //--------------------------------
  // Init vom Timer-0 (8bit)
  // erzeugt das PWM-Signal fuer Kanal links und rechts
  // Ausgang = OC0A = PB7 (links)
  //           OC0B = PG5 (rechts)
  //--------------------------------
  TCCR0A=0xA1; // Mode = 1 (PWM mit OC0A+OC0B, Top=0xFF)
  TCNT0=0x00;  // startwert
  OCR0A=0x00;  // vergleichswert (pwm)
  OCR0B=0x00;  // vergleichswert (pwm)

  //--------------------------------
  // Init vom Timer-1 (16bit)
  // eingestellt auf die Abtastrate
  //--------------------------------
  TCCR1A=0x00;  // Mode = 4 (CTC, Top=OC1A)
  TCCR1C=0x00;  // nc
  TCNT1=0x0000; // startwert
  OCR1A=2000;   // vergleichswert (samplerate)

  //--------------------------------
  // Interruptfreigabe
  //--------------------------------
  TIMSK1=0x02;  // Timer1 Comp-A Interrupt enable
  SREG|=0x80;   // Alle Interrupts einschalten
}

//-----------------------------------------------------------------------------
WAV_CHK MK3_WAV_LOAD(const char *ptr) { // zum laden eines WAV-Files
  WAV_CHK ret_wert=WAV_LOAD_OK;

  // 1. Versuch = normaler PCM-Typ
  ret_wert=WAV_LOAD_FILE(ptr,WAV_PCM_NORMAL);
  if(ret_wert==WAV_LOAD_OK) return(ret_wert);

  // 2. Versuch = PCM mit "fact-chunk"
  ret_wert=WAV_LOAD_FILE(ptr,WAV_PCM_FACT);  

  return(ret_wert);
}


//-----------------------------------------------------------------------------
void MK3_WAV_START(const char *ptr) {               // zum starten eines WAV-Files
  if(WAV_HEADER.header_ok==true) {
    // nur wenn header ok
    if(WAV_HEADER.status==WAV_PAUSE)  {
      // bei PAUSE

      WAV_HEADER.status=WAV_PLAY;

      // Timer starten
      TCCR0B=0x01;  // Frq = Quarz/1
      TCCR1B=0x09;  // Frq = Quarz/1       
    }
    else if(WAV_HEADER.status==WAV_STOP) {
      // bei STOP neu laden
      if(MK3_WAV_LOAD(ptr)==WAV_LOAD_OK) {
        WAV_HEADER.status=WAV_PLAY;

        // Timer starten
        TCCR0B=0x01;  // Frq = Quarz/1
        TCCR1B=0x09;  // Frq = Quarz/1     
      }
    }
  }
}


//-----------------------------------------------------------------------------
void MK3_WAV_PAUSE(void) {               // zum pausieren eines WAV-Files
  if((WAV_HEADER.header_ok==true) && (WAV_HEADER.status==WAV_PLAY))  {
    // nur wenn header ok ist und PLAY

    WAV_HEADER.status=WAV_PAUSE;

    // Timer stoppen
    TCCR0B=0x00;  // Frq = stop
    TCCR1B=0x08;  // Frq = stop
    TCNT0=0x00;   // startwert
    TCNT1=0x0000; // startwert
    OCR0A=0; 
    OCR0B=0;    
  }
}


//-----------------------------------------------------------------------------
void MK3_WAV_STOP(void) {               // zum stopen eines WAV-Files
  if((WAV_HEADER.header_ok==true) && (WAV_HEADER.status!=WAV_STOP))  {
    // nur wenn header ok ist und nicht STOP

    WAV_HEADER.status=WAV_STOP;

    // Timer stoppen
    TCCR0B=0x00;  // Frq = stop
    TCCR1B=0x08;  // Frq = stop
    TCNT0=0x00;   // startwert
    TCNT1=0x0000; // startwert
    OCR0A=0;  
    OCR0B=0;

    WAV_DATA.file_stop=true;  // merker setzen   
  }
}


//-----------------------------------------------------------------------------
// diese Funktion muss staendig vom Hauptprogramm aus aufgerufen werden
// also direkt von der while(1) 
// die Aufrufgeschwindigkeit muss schnell genug sein, sonnst kommt es
// zu knistern und stottern im Ton
//-----------------------------------------------------------------------------
void MK3_WAV_DO(void) {
  intptr_t count;
  unsigned int n;
  
  // test ob puffer nachgeladen werden muss
  if(WAV_DATA.file_ende==false) {
    // wenn noch nicht das file-ende erreicht ist
    if(WAV_DATA.load_buffer==true) {
      WAV_DATA.load_buffer=false;

      if(WAV_DATA.akt_buffer==0) {
        count = fat_read_file(MK3_SD.file, WAV_DATA.buffer[1], WAV_BUFFER_LEN);
        if(count==0) WAV_DATA.file_stop=true;
      }
      else {
        count = fat_read_file(MK3_SD.file, WAV_DATA.buffer[0], WAV_BUFFER_LEN);
        if(count==0) WAV_DATA.file_stop=true;
      }
    }
  }

  // test ob file gestoppt werden soll
  if(WAV_DATA.file_stop==true) {
    WAV_DATA.file_stop=false;
    WAV_DATA.file_ende=true;
    MK3_SD_CLOSE_FILE();

    WAV_HEADER.status=WAV_STOP;

    // Timer stoppen
    TCCR0B=0x00;  // Frq = stop
    TCCR1B=0x08;  // Frq = stop
    TCNT0=0x00;   // startwert
    TCNT1=0x0000; // startwert
    OCR0A=0;  
    OCR0B=0;

    // Daten loeschen
    for(n=0;n<WAV_BUFFER_LEN;n++) {
      WAV_DATA.buffer[0][n]=0;
      WAV_DATA.buffer[1][n]=0;
    }
    WAV_DATA.akt_pos=0;
    WAV_DATA.akt_buffer=0;
 
    WAV_HEADER.status=WAV_STOP; 
  }
}




//-----------------------------------------------------------------------------
// interne Funktion zum laden eines WAV-Files
// mit einem bestimmten Typ
//-----------------------------------------------------------------------------
WAV_CHK WAV_LOAD_FILE(const char *ptr, WAV_TYP typ) {	
  WAV_CHK ret_wert=WAV_LOAD_OK;
  FFR_Status_t check;
  unsigned int n;
  intptr_t count;
  unsigned int header_len=0;

  // test auf Status
  if(WAV_HEADER.status==WAV_STOP) {
    // WAV-File laden
    check=MK3_SD_READ_FILE_IN_ROOT(ptr);
    if(check==SDR_READ_OK) {
      // laden war ok
      // WAV-Header laden
      if(typ==WAV_PCM_NORMAL) header_len=44;
      if(typ==WAV_PCM_FACT) header_len=58;
      count = fat_read_file(MK3_SD.file, WAV_DATA.buffer[0], header_len);
      if(count==header_len) {
        // WAV-Header testen
        WAV_CHK_HEADER(typ);
        if(WAV_HEADER.header_ok==true) {
          // alles ok
          ret_wert=WAV_LOAD_OK;
          // status auf Pause setzen
          // damit bei START nicht nochmal geladen wird
          WAV_HEADER.status=WAV_PAUSE;
          // beide puffer mit daten laden
          count = fat_read_file(MK3_SD.file, WAV_DATA.buffer[0], WAV_BUFFER_LEN);
          if(count==0) {
            WAV_DATA.file_ende=true;
          }
          else {
            count = fat_read_file(MK3_SD.file, WAV_DATA.buffer[1], WAV_BUFFER_LEN);
            if(count==0) {
              WAV_DATA.file_ende=true;
            }
          }
        }
        else {
          // fehler im header
          ret_wert=WAV_HEADER_ERR;
          MK3_SD_CLOSE_FILE();
        }
      }
      else {
        // falsche anzahl von daten gelesen
        ret_wert=WAV_HEADER_LEN_ERR;
        MK3_SD_CLOSE_FILE();
      }
    }
    else {
      // fehler beim laden
      ret_wert=WAV_NO_SD;
      if(check==SDR_NO_CARD) ret_wert=WAV_NO_SD;
      if(check==SDR_PARTITION_ERR) ret_wert=WAV_PARTITION_ERR;
      if(check==SDR_FAT_ERR) ret_wert=WAV_FAT_ERR;
      if(check==SDR_DIR_ERR) ret_wert=WAV_DIR_ERR;
      if(check==SDR_FILE_NOT_FOUND) ret_wert=WAV_FILE_NOT_FOUND;
    }
  }
  else {
    // status steht nicht auf STOP
    ret_wert=WAV_NO_STOP;
  }

  return(ret_wert);  
}

//-----------------------------------------------------------------------------
// Testet die Daten vom WAV-Header auf Kompatibilitaet
// Untersttzte Formate :
//  WAV_PCM_NORMAL = Typ = 0x0001 = PCM (normal)
//  WAV_PCM_FACT   = Typ = 0x0001 = PCM (mit "fact-Chunk")
//-----------------------------------------------------------------------------
void WAV_CHK_HEADER(WAV_TYP typ) {  // interne Funktion zum Check des Headers
  unsigned char d_lo=0,d_hi=0;
  unsigned int uint=0;
  unsigned int offset=0; // offset fuer fact-chunk

  // Header loeschen
  WAV_HEADER.header_ok=false;
  WAV_HEADER.err_nr=0;
  WAV_HEADER.fmt_tag=0;
  WAV_HEADER.kanal=0;
  WAV_HEADER.sampl=0;
  WAV_HEADER.frame=0;
  WAV_HEADER.bits=0;
  WAV_HEADER.len=0;
  WAV_HEADER.mode=WAV_1MONO;
  WAV_HEADER.status=WAV_STOP;

  // Daten loeschen
  WAV_DATA.akt_pos=0;
  WAV_DATA.akt_buffer=0;
  WAV_DATA.load_buffer=false;
  WAV_DATA.file_ende=false;
  WAV_DATA.file_stop=false;

  // [1]=RIFF-Kennung auslesen [0x00 - 0x03] (4 Bytes)
  if(WAV_HEADER.err_nr==0) {
    // Sollwert = 'RIFF'
    d_lo=WAV_DATA.buffer[0][0x00];
    if(d_lo!=0x52) WAV_HEADER.err_nr=1;
    d_lo=WAV_DATA.buffer[0][0x01];
    if(d_lo!=0x49) WAV_HEADER.err_nr=1;
    d_lo=WAV_DATA.buffer[0][0x02];
    if(d_lo!=0x46) WAV_HEADER.err_nr=1;
    d_lo=WAV_DATA.buffer[0][0x03];
    if(d_lo!=0x46) WAV_HEADER.err_nr=1;
  }

  // [2]=Dateigroesse auslesen [0x04 - 0x07] (4 Bytes,nc)
  if(WAV_HEADER.err_nr==0) {
    // Sollwert = egal
    d_lo=WAV_DATA.buffer[0][0x04];
    d_lo=WAV_DATA.buffer[0][0x05];
    d_lo=WAV_DATA.buffer[0][0x06];
    d_lo=WAV_DATA.buffer[0][0x07];
  }

  // [3]=WAVE-Kennung auslesen [0x08 - 0x0b] (4 Bytes)
  if(WAV_HEADER.err_nr==0) {
    // Sollwert = 'WAVE'
    d_lo=WAV_DATA.buffer[0][0x08];
    if(d_lo!=0x57) WAV_HEADER.err_nr=3; 
    d_lo=WAV_DATA.buffer[0][0x09];
    if(d_lo!=0x41) WAV_HEADER.err_nr=3; 
    d_lo=WAV_DATA.buffer[0][0x0a];
    if(d_lo!=0x56) WAV_HEADER.err_nr=3; 
    d_lo=WAV_DATA.buffer[0][0x0b];
    if(d_lo!=0x45) WAV_HEADER.err_nr=3; 
  }

  // [4]=fmt-Kennung auslesen [0x0c - 0x0f] (4 Bytes)
  if(WAV_HEADER.err_nr==0) {
    // Sollwert = 'fmt '
    d_lo=WAV_DATA.buffer[0][0x0c];
    if(d_lo!=0x66) WAV_HEADER.err_nr=4; 
    d_lo=WAV_DATA.buffer[0][0x0d];
    if(d_lo!=0x6d) WAV_HEADER.err_nr=4; 
    d_lo=WAV_DATA.buffer[0][0x0e];
    if(d_lo!=0x74) WAV_HEADER.err_nr=4; 
    d_lo=WAV_DATA.buffer[0][0x0f];
    if(d_lo!=0x20) WAV_HEADER.err_nr=4; 
  }   

  // [5]=fmt laenge auslesen [0x10 - 0x13] (4 Bytes)
  if(WAV_HEADER.err_nr==0) {
    // Sollwert = 16 dez (im Normalfall)
    // Sollwert = 18 dez (mit "fact-chunk")
    d_lo=WAV_DATA.buffer[0][0x10];
    d_hi=WAV_DATA.buffer[0][0x11];
    uint=(d_hi<<8)|(d_lo);
    if((uint==16) && (typ==WAV_PCM_NORMAL)) {
      // normalfall
      offset=0;
    }
    else if((uint==18) && (typ==WAV_PCM_FACT)) {
      // mit "fact-chunk"
      offset=14;
    }
    else {
      // nicht erlaubt
      WAV_HEADER.err_nr=5;
    }
    d_lo=WAV_DATA.buffer[0][0x12];
    d_hi=WAV_DATA.buffer[0][0x13];
    uint=(d_hi<<8)|(d_lo);    
    if(uint!=0) WAV_HEADER.err_nr=5;
  }

  // [6]=format TAG auslesen [0x14 - 0x15] (2 Bytes) 
  if(WAV_HEADER.err_nr==0) {
    // Sollwert = 1 dez (Typ = PCM)
    d_lo=WAV_DATA.buffer[0][0x14];
    d_hi=WAV_DATA.buffer[0][0x15];
    uint=(d_hi<<8)|(d_lo);
    WAV_HEADER.fmt_tag=uint;
    if(uint!=1) WAV_HEADER.err_nr=6;
  }   

  // [7]=anzahl der kanaele auslesen [0x16 - 0x17] (2 Bytes) 
  if(WAV_HEADER.err_nr==0) {
    // Sollwert = 1 oder 2
    d_lo=WAV_DATA.buffer[0][0x16];
    d_hi=WAV_DATA.buffer[0][0x17];
    uint=(d_hi<<8)|(d_lo);
    WAV_HEADER.kanal=uint;    
    if((WAV_HEADER.kanal<1) || (WAV_HEADER.kanal>2)) {
      WAV_HEADER.err_nr=7;
    }
  }

  // [8]=sample rate auslesen [0x18 - 0x1b] (4 Bytes) 
  if(WAV_HEADER.err_nr==0) {
    // Sollwert <= Maxsamplerate
    d_lo=WAV_DATA.buffer[0][0x18];
    d_hi=WAV_DATA.buffer[0][0x19];
    uint=(d_hi<<8)|(d_lo);
    WAV_HEADER.sampl=uint;
    if(uint>WAV_MAX_SAMPLE) WAV_HEADER.err_nr=8;
    d_lo=WAV_DATA.buffer[0][0x1a];
    d_hi=WAV_DATA.buffer[0][0x1b];
    uint=(d_hi<<8)|(d_lo);
    if(uint!=0) WAV_HEADER.err_nr=8;
  }

  // [9]=abtast rate auslesen [0x1c - 0x1f] (4 Bytes,nc)
  if(WAV_HEADER.err_nr==0) {
    // Sollwert = egal
    d_lo=WAV_DATA.buffer[0][0x1c];
    d_lo=WAV_DATA.buffer[0][0x1d];
    d_lo=WAV_DATA.buffer[0][0x1e];
    d_lo=WAV_DATA.buffer[0][0x1f];
  }

  // [10]=frame groesse auslesen [0x20 - 0x21] (2 Bytes) 
  if(WAV_HEADER.err_nr==0) {
    // Sollwert = 1 oder 2 oder 4
    d_lo=WAV_DATA.buffer[0][0x20];
    d_hi=WAV_DATA.buffer[0][0x21];
    uint=(d_hi<<8)|(d_lo);
    WAV_HEADER.frame=uint;    
    if((WAV_HEADER.frame<1) || (WAV_HEADER.frame>4)) {
      WAV_HEADER.err_nr=10;
    }
    if(WAV_HEADER.frame==3) WAV_HEADER.err_nr=10;
  }

  // [11]=bits pro sample auslesen [0x22 - 0x23] (2 Bytes) 
  if(WAV_HEADER.err_nr==0) {
    // Sollwert = 1 bis 8 oder 16
    d_lo=WAV_DATA.buffer[0][0x22];
    d_hi=WAV_DATA.buffer[0][0x23];
    uint=(d_hi<<8)|(d_lo);
    WAV_HEADER.bits=uint;    
    if((WAV_HEADER.bits<1) || (WAV_HEADER.bits>16)) {
      WAV_HEADER.err_nr=11;
    }
    if((WAV_HEADER.bits>8) && (WAV_HEADER.bits!=16)) {
      WAV_HEADER.err_nr=11;
    }
  } 

  // [12]=data-Kennung auslesen [0x24 - 0x27] (4 Bytes)
  if(WAV_HEADER.err_nr==0) {
    // Sollwert = 'data'
    d_lo=WAV_DATA.buffer[0][0x24+offset];
    if(d_lo!=0x64) WAV_HEADER.err_nr=12; 
    d_lo=WAV_DATA.buffer[0][0x25+offset];
    if(d_lo!=0x61) WAV_HEADER.err_nr=12; 
    d_lo=WAV_DATA.buffer[0][0x26+offset];
    if(d_lo!=0x74) WAV_HEADER.err_nr=12; 
    d_lo=WAV_DATA.buffer[0][0x27+offset];
    if(d_lo!=0x61) WAV_HEADER.err_nr=12; 
  }

  // [13]=daten laenge auslesen [0x28 - 0x2b] (4 Bytes) 
  if(WAV_HEADER.err_nr==0) {
    // Sollwert = egal
    d_lo=WAV_DATA.buffer[0][0x2a+offset];
    d_hi=WAV_DATA.buffer[0][0x2b+offset];
    uint=(d_hi<<8)|(d_lo);
    WAV_HEADER.len=uint;
    WAV_HEADER.len=(WAV_HEADER.len<<16); 
    d_lo=WAV_DATA.buffer[0][0x28+offset];
    d_hi=WAV_DATA.buffer[0][0x29+offset];
    uint=(d_hi<<8)|(d_lo); 
    WAV_HEADER.len|=uint;   
  }  

  // [14]=plausibilitaetstest
  if(WAV_HEADER.err_nr==0) { 
    // Sollwert = es gibt nur 4 erlaubte Kombinationen
    // 1 = Mono,   8bit,  1Frame
    // 2 = Stereo, 8bit,  2Frames
    // 3 = Mono,   16bit, 2Frames
    // 4 = Stereo, 16bit, 4Frames
    if((WAV_HEADER.kanal==1) && (WAV_HEADER.bits<=8)) {
      if(WAV_HEADER.frame!=1) WAV_HEADER.err_nr=14;
    }
    if((WAV_HEADER.kanal==2) && (WAV_HEADER.bits<=8)) {
      if(WAV_HEADER.frame!=2) WAV_HEADER.err_nr=14;
    }
    if((WAV_HEADER.kanal==1) && (WAV_HEADER.bits>8)) {
      if(WAV_HEADER.frame!=2) WAV_HEADER.err_nr=14;
    }
    if((WAV_HEADER.kanal==2) && (WAV_HEADER.bits>8)) {
      if(WAV_HEADER.frame!=4) WAV_HEADER.err_nr=14;
    }
  }

  // Mode setzen falls alles ok
  if(WAV_HEADER.err_nr==0) {    
    if(WAV_HEADER.kanal==1) {
      // mono
      if(WAV_HEADER.frame==1) {
        WAV_HEADER.mode=WAV_1MONO;
      }
      else {
        WAV_HEADER.mode=WAV_2MONO;
      }
    }
    else {
      // stereo
      if(WAV_HEADER.frame==2) {
        WAV_HEADER.mode=WAV_1STEREO;
      }
      else {
        WAV_HEADER.mode=WAV_2STEREO;
      }
    }
  }

  // Samplerate vom Timer einstellen falls alles ok
  if(WAV_HEADER.err_nr==0) {
    OCR1A=WAV_QUARZ_FRQ/WAV_HEADER.sampl;
  }

  // Header-Status setzen falls alles ok
  if(WAV_HEADER.err_nr==0) {
    WAV_HEADER.header_ok=true;
  }
}




//-----------------------------------------------------------------------------
// stellt die neuen PWM-Werte ein
// fuer links und rechts
// liest die Daten aus dem Puffer
//-----------------------------------------------------------------------------
ISR (TIMER1_COMPA_vect) {                          // ISR : Timer1 Comp-A
  unsigned char data_lo,data_hi;
  unsigned char data_left,data_right;

  //---------------------------------------------
  // Daten aus dem Puffer laden 
  // und in PWM-links und PWM-rechts wandeln
  // 16bit werden in 8bit umgerechnet
  //---------------------------------------------
  switch(WAV_HEADER.mode) {
    case WAV_1MONO : // MONO-Signal aus einem Frame (8bit)
      data_left=WAV_DATA.buffer[WAV_DATA.akt_buffer][WAV_DATA.akt_pos];
      data_right=data_left;				
    ;break;
    case WAV_2MONO : // MONO-Signal aus zwei Frames (16bit)
      // data lo wird nicht benutzt
//      data_lo=WAV_DATA.buffer[WAV_DATA.akt_buffer][WAV_DATA.akt_pos];
      data_hi=WAV_DATA.buffer[WAV_DATA.akt_buffer][WAV_DATA.akt_pos+1];
      if(data_hi<=0x7F) {
        // positive Zahl (MSB ist nicht gesetzt)
        data_left=data_hi+128;
      }
      else {
        // negative Zahl (MSB ist gesetzt)
        data_left=data_hi-128;
      }
      data_right=data_left;
    ;break;
    case WAV_1STEREO : // Stereo-Signal aus je einem Frame (8bit)
      data_left=WAV_DATA.buffer[WAV_DATA.akt_buffer][WAV_DATA.akt_pos];
      data_right=WAV_DATA.buffer[WAV_DATA.akt_buffer][WAV_DATA.akt_pos+1];
    ;break;
    case WAV_2STEREO : // Stereo-Signal aus je zwei Frames (16bit)
      // data lo wird nicht benutzt
//      data_lo=WAV_DATA.buffer[WAV_DATA.akt_buffer][WAV_DATA.akt_pos];
      data_hi=WAV_DATA.buffer[WAV_DATA.akt_buffer][WAV_DATA.akt_pos+1];
      if(data_hi<=0x7F) {
        // positive Zahl (MSB ist nicht gesetzt)
        data_left=data_hi+128;
      }
      else {
        // negative Zahl (MSB ist gesetzt)
        data_left=data_hi-128;
      }
      // data lo wird nicht benutzt
//      data_lo=WAV_DATA.buffer[WAV_DATA.akt_buffer][WAV_DATA.akt_pos+2];
      data_hi=WAV_DATA.buffer[WAV_DATA.akt_buffer][WAV_DATA.akt_pos+3];
      if(data_hi<=0x7F) {
        // positive Zahl (MSB ist nicht gesetzt)
        data_right=data_hi+128;
      }
      else {
        // negative Zahl (MSB ist gesetzt)
        data_right=data_hi-128;
      }
    ;break;
  }

  // Pointer weiterschalten
  WAV_DATA.akt_pos+=WAV_HEADER.frame;

  // test ob puffer fertig abgearbeitet wurde
  if(WAV_DATA.akt_pos>=WAV_BUFFER_LEN) {
    WAV_DATA.akt_pos=0;
    WAV_DATA.akt_buffer=1-WAV_DATA.akt_buffer;
    WAV_DATA.load_buffer=true; // merker setzen
  }

  // neue PWM-Daten ausgeben
  OCR0A=data_left; 
  OCR0B=data_right;
}



